@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*--- Read stabs debug info.                     vg_symtab_stabs.c ---*/
+/*--- Read stabs debug info.                            vg_stabs.c ---*/
 /*--------------------------------------------------------------------*/
 
 /*
@@ -27,7 +27,7 @@
    The GNU General Public License is contained in the file COPYING.
 */
 
-#include "vg_include.h"
+#include "core.h"
 #include "vg_symtab2.h"
 
 #include <a.out.h>        /* stabs defns                    */
@@ -101,6 +101,7 @@ typedef struct _StabTypeTab {
    }		*headerhash[HEADER_HASHSZ];
 } StabTypeTab;
 
+static const Bool stabs_debug = False;
 
 static UInt header_hash(Char *filename, UInt instance)
 {
@@ -122,7 +123,7 @@ static UInt header_hash(Char *filename, UInt instance)
 */
 static SymType *structRef(StabTypeTab *tab, SymType *def, Bool isstruct, Char *name)
 {
-   static const Bool debug = False;
+   static const Bool debug = False || stabs_debug;
    struct structlist *sl;
    SymType *ty;
    static Int warnlen = 0;
@@ -167,15 +168,25 @@ static SymType *structRef(StabTypeTab *tab, SymType *def, Bool isstruct, Char *n
 /* Add a structural defintion for a struct/union reference */
 static SymType *structDef(StabTypeTab *tab, SymType *def, Bool isstruct, Char *name)
 {
-   static const Bool debug = False;
+   static const Bool debug = False || stabs_debug;
    SymType *ref = structRef(tab, NULL, isstruct, name);
 
-   if (debug)
-      VG_(printf)("defining %s ref for %s %p -> %p\n",
-		  isstruct ? "struct" : "union", name, ref, def);
+   /* it seems that GNAT likes to declare names as both struct tags
+      and typedefs so check we aren't about to make a structure a
+      reference to itself as that will create a loop */
+   if (ref == def) {
+      if (debug)
+         VG_(printf)("ignoring %s self ref for %s %p -> %p\n",
+		     isstruct ? "struct" : "union", name, ref, def);
+   }
+   else {
+      if (debug)
+         VG_(printf)("defining %s ref for %s %p -> %p\n",
+		     isstruct ? "struct" : "union", name, ref, def);
 
-   def = VG_(st_mktypedef)(ref, name, VG_(st_basetype)(def, False));
-   VG_(st_setname)(def, name);
+      def = VG_(st_mktypedef)(ref, name, VG_(st_basetype)(def, False));
+      VG_(st_setname)(def, name);
+   }
    return def;
 }
 
@@ -225,7 +236,7 @@ static StabFile *getStabFile(StabTypeTab *tab, Int file, StabFile *set)
 /* add a new index for a file */
 static void addFileAlias(StabTypeTab *tab, Char *filename, UInt instance, Int idx)
 {
-   static const Bool debug = False;
+   static const Bool debug = False || stabs_debug;
    struct header *hp;
 
    for(hp = tab->headerhash[header_hash(filename, instance)]; hp != NULL; hp = hp->next) {
@@ -244,7 +255,7 @@ static void addFileAlias(StabTypeTab *tab, Char *filename, UInt instance, Int id
 
 static void addHeader(StabTypeTab *tab, Char *filename, UInt instance, Int idx)
 {
-   static const Bool debug = False;
+   static const Bool debug = False || stabs_debug;
    struct header *hp, **bucket;
    
    if (debug)
@@ -423,8 +434,11 @@ static Char *templ_name(Char *p)
        */
       if (brac && p[0] == '\'' && p[2] == '\'')
 	 p += 3;
+
+      /* If we're within <>, then treat :: as part of the name (a single
+	 : still terminates) */
       if (*p == ':') {
-	 if (brac && p[1] == ':')
+	 if (brac && p[1] == ':' && p[-1] != '<')
 	    p++;
 	 else
 	    break;
@@ -462,7 +476,7 @@ static void parse_typeref(Char **pp, Int *filep, Int *symp)
 
 static void stab_resolve(SymType *st, void *data)
 {
-   static const Bool debug = False;
+   static const Bool debug = False || stabs_debug;
    Char *str = (Char *)data;
    vg_assert(!VG_(st_isresolved)(st));
 
@@ -479,7 +493,7 @@ static void stab_resolve(SymType *st, void *data)
    introduced anywhere, so we need to scan it all to pick them up. */
 static SymType *stabtype_parser(SegInfo *si, SymType *def, Char **pp)
 {
-   static const Bool debug = False;
+   static const Bool debug = False || stabs_debug;
    Char *p = *pp;
    Char t;
    SymType *type;
@@ -607,14 +621,47 @@ static SymType *stabtype_parser(SegInfo *si, SymType *def, Char **pp)
       p--;
       n = atoi(&p, 0);
       switch(n) {
-      case -16:	type = VG_(st_mkbool)(def, sizeof(int)); break;
+      case -1:	type = VG_(st_mkint)(def, 4, True); break;
+      case -2:	type = VG_(st_mkint)(def, 1, True); break;
+      case -3:	type = VG_(st_mkint)(def, 2, True); break;
+      case -4:	type = VG_(st_mkint)(def, 4, True); break;
+      case -5:	type = VG_(st_mkint)(def, 1, False); break;
+      case -6:	type = VG_(st_mkint)(def, 1, True); break;
+      case -7:	type = VG_(st_mkint)(def, 2, False); break;
+      case -8:	type = VG_(st_mkint)(def, 4, False); break;
+      case -9:	type = VG_(st_mkint)(def, 4, False); break;
+      case -10:	type = VG_(st_mkint)(def, 4, False); break;
+      case -11:	type = VG_(st_mkvoid)(def); break;
+      case -12:	type = VG_(st_mkfloat)(def, 4); break;
+      case -13:	type = VG_(st_mkfloat)(def, 8); break;
+      case -15:	type = VG_(st_mkint)(def, 4, True); break;
+      case -16:	type = VG_(st_mkbool)(def, 4); break;
+      case -17:	type = VG_(st_mkfloat)(def, 4); break;
+      case -18:	type = VG_(st_mkfloat)(def, 8); break;
+      case -20:	type = VG_(st_mkint)(def, 1, False); break;
+      case -21:	type = VG_(st_mkint)(def, 1, False); break;
+      case -22:	type = VG_(st_mkint)(def, 2, False); break;
+      case -23:	type = VG_(st_mkint)(def, 4, False); break;
+      case -24:	type = VG_(st_mkint)(def, 4, False); break;
+      case -27:	type = VG_(st_mkint)(def, 1, True); break;
+      case -28:	type = VG_(st_mkint)(def, 2, True); break;
+      case -29:	type = VG_(st_mkint)(def, 4, True); break;
+      case -31:	type = VG_(st_mkint)(def, 8, True); break;
+      case -32:	type = VG_(st_mkint)(def, 8, False); break;
+      case -33:	type = VG_(st_mkint)(def, 8, False); break;
+      case -34:	type = VG_(st_mkint)(def, 8, True); break;
 
       default:
 	 VG_(printf)(" @@ unrecognized negative type %d\n", n);
 	 type = NULL;
 	 break;
       }
-      EXPECT(';', "negative builtin type");
+      /* Different versions of gcc seem to disagree about whether a
+         negative type is followed by a semicolon or not, and the stabs
+         spec (susch as it is) is not clear either so we will skip a
+         semicolon if there is one. */
+      if (*p == ';')
+         p++;
       break;
    }
 
@@ -631,10 +678,8 @@ static SymType *stabtype_parser(SegInfo *si, SymType *def, Char **pp)
       EXPECT(';', "FP-TYPE");
       bytes = atoi(&p, 0);
       EXPECT(';', "FP-TYPE bytes");
-      atoi(&p, 0);
-      EXPECT(';', "FP-TYPE extra");
       
-      type = VG_(st_mkfloat)(def, bytes * 8);
+      type = VG_(st_mkfloat)(def, bytes);
       break;
    }
 
@@ -802,7 +847,7 @@ static SymType *stabtype_parser(SegInfo *si, SymType *def, Char **pp)
 	 UInt off, sz;
 	 SymType *fieldty;
 
-	 end = templ_name(p);
+         end = SKIPPAST(p, ':', "method name") - 1;
 
 	 if (end[1] == ':') {
 	    /* c++ method names end in :: */
@@ -873,7 +918,10 @@ static SymType *stabtype_parser(SegInfo *si, SymType *def, Char **pp)
 	    } else {
 	       EXPECT(',', "struct TYPE");
 
-	       off = atou(&p, 0);
+               /* logic dictates that the offset would always be
+	          positive and that atou would work here but GNAT has
+	          has other ideas - see bug 90128 for more details */
+	       off = atoi(&p, 0);
 
 	       if (*p == ',') {
 		  EXPECT(',', "struct OFFSET");
@@ -968,7 +1016,7 @@ static SymType *stabtype_parser(SegInfo *si, SymType *def, Char **pp)
 /* parse a symbol reference: NAME ':' DESC TYPE */
 static Bool initSym(SegInfo *si, Sym *sym, stab_types kind, Char **namep, Int val)
 {
-   static const Bool debug = False;
+   static const Bool debug = False || stabs_debug;
    Char *name = *namep;
    Char *ty;
    Int len;
@@ -980,7 +1028,18 @@ static Bool initSym(SegInfo *si, Sym *sym, stab_types kind, Char **namep, Int va
       VG_(printf)("initSym(si=%p, tab=%p, sym=%p, kind=%d, name=%p \"%s\", val=%d)\n",
 		  si, si->stab_typetab, sym, kind, name, name, val);
 
-   ty = templ_name(name);
+   /* Find first ':' */
+   ty = VG_(strchr)(name, ':');
+
+   /* Skip '::' */
+   while (ty && ty[1] == ':')
+      ty = VG_(strchr)(ty + 2, ':');
+
+   if (ty == NULL) {
+      /* there was no ':' */
+      *namep += VG_(strlen)(name);
+      return True;		/* skip */
+   }
 
    len = ty - name;
 
@@ -1026,7 +1085,6 @@ static Bool initSym(SegInfo *si, Sym *sym, stab_types kind, Char **namep, Int va
      out:
       sym->type = stabtype_parser(si, NULL, &ty);
       base = VG_(st_basetype)(sym->type, False);
-
       if (isStruct && (VG_(st_isstruct)(base) || VG_(st_isunion)(base))) {
 	 Char *sname = VG_(addStr)(si, name, len);
 	 structDef(si->stab_typetab, base, VG_(st_isstruct)(base), sname);
@@ -1099,7 +1157,7 @@ struct symlist {
    definitions helps a lot. */
 static Scope *addSymsToScope(Scope *sc, struct symlist *list, Int nsyms, Scope *outer)
 {
-   static const Bool debug = False;
+   static const Bool debug = False || stabs_debug;
    Int j;
    struct symlist *n;
    Int base;
@@ -1145,7 +1203,7 @@ void VG_(read_debuginfo_stabs) ( SegInfo* si,
 				 UChar* stabC,   Int stab_sz, 
 				 UChar* stabstr, Int stabstr_sz )
 {
-   static const Bool debug = False;
+   static const Bool debug = False || stabs_debug;
    Int    i;
    Int    n_stab_entries;
    struct nlist* stab = (struct nlist*)stabC;
@@ -1223,7 +1281,7 @@ void VG_(read_debuginfo_stabs) ( SegInfo* si,
 
       /* handle continued string stabs */
       {
-	 static const Bool contdebug = False;
+	 static const Bool contdebug = False || stabs_debug;
 	 Int buflen = 0;
 	 Int idx = 0;
 	 Char *buf = NULL;
@@ -1637,5 +1695,5 @@ void VG_(read_debuginfo_stabs) ( SegInfo* si,
 }
 
 /*--------------------------------------------------------------------*/
-/*--- end                                        vg_symtab_stabs.c ---*/
+/*--- end                                                          ---*/
 /*--------------------------------------------------------------------*/
