@@ -8,7 +8,7 @@
    This file is part of Helgrind, a Valgrind tool for detecting
    data races in threaded programs.
 
-   Copyright (C) 2002-2004 Nicholas Nethercote
+   Copyright (C) 2002-2005 Nicholas Nethercote
       njn25@cam.ac.uk
 
    This program is free software; you can redistribute it and/or
@@ -29,7 +29,7 @@
    The GNU General Public License is contained in the file COPYING.
 */
 
-#include "vg_skin.h"
+#include "tool.h"
 #include "helgrind.h"
 
 static UInt n_eraser_warnings = 0;
@@ -142,7 +142,7 @@ typedef
    struct _HG_Chunk {
       struct _HG_Chunk* next;
       Addr          data;           /* ptr to actual block              */
-      Int           size;           /* size requested                   */
+      SizeT         size;           /* size requested                   */
       ExeContext*   where;          /* where it was allocated           */
       ThreadId      tid;            /* allocating thread                */
    }
@@ -224,24 +224,24 @@ static const shadow_word error_sword = SW(Vge_Excl, TLSP_INDICATING_ALL);
 /* Parallel map which contains execution contexts when words last
   changed state (if required) */
 
-typedef struct EC_EIP {
-   union u_ec_eip {
-      Addr		eip;
+typedef struct EC_IP {
+   union u_ec_ip {
+      Addr		ip;
       ExeContext	*ec;
-   } uu_ec_eip;
+   } uu_ec_ip;
    UInt			state:STATE_BITS;
    UInt			tls:OTHER_BITS;		/* packed TLS */
-} EC_EIP;
+} EC_IP;
 
-#define NULL_EC_EIP	((EC_EIP){ { 0 }, 0, 0})
+#define NULL_EC_IP	((EC_IP){ { 0 }, 0, 0})
 
-#define EIP(eip, prev, tls) ((EC_EIP) { (union u_ec_eip)(eip), (prev).state, packTLS(tls) })
-#define EC(ec, prev, tls)   ((EC_EIP) { (union u_ec_eip)(ec), (prev).state, packTLS(tls) })
+#define IP(ip, prev, tls) ((EC_IP) { (union u_ec_ip)(ip), (prev).state, packTLS(tls) })
+#define EC(ec, prev, tls)   ((EC_IP) { (union u_ec_ip)(ec), (prev).state, packTLS(tls) })
 
 static inline UInt packEC(ExeContext *ec)
 {
-   SK_ASSERT(((UInt)ec & ((1 << STATE_BITS)-1)) == 0);
-   return ((UInt)ec) >> STATE_BITS;
+   SK_ASSERT(((UWord)ec & ((1 << STATE_BITS)-1)) == 0);
+   return ((UWord)ec) >> STATE_BITS;
 }
 
 static inline ExeContext *unpackEC(UInt i)
@@ -249,24 +249,24 @@ static inline ExeContext *unpackEC(UInt i)
    return (ExeContext *)(i << STATE_BITS);
 }
 
-/* Lose 2 LSB of eip */
-static inline UInt packEIP(Addr eip)
+/* Lose 2 LSB of IP */
+static inline UInt packIP(Addr ip)
 {
-   return ((UInt)eip) >> STATE_BITS;
+   return ip >> STATE_BITS;
 }
 
-static inline Addr unpackEIP(UInt i)
+static inline Addr unpackIP(UInt i)
 {
    return (Addr)(i << STATE_BITS);
 }
 
 typedef struct {
-   EC_EIP execontext[ESEC_MAP_WORDS];
+   EC_IP execontext[ESEC_MAP_WORDS];
 } ExeContextMap;
 
 static ExeContextMap** execontext_map;
 
-static inline void setExeContext(Addr a, EC_EIP ec)
+static inline void setExeContext(Addr a, EC_IP ec)
 {
    UInt idx = (a >> 16) & 0xffff;
    UInt off = (a >>  2) & 0x3fff;
@@ -279,11 +279,11 @@ static inline void setExeContext(Addr a, EC_EIP ec)
    execontext_map[idx]->execontext[off] = ec;
 }
 
-static inline EC_EIP getExeContext(Addr a)
+static inline EC_IP getExeContext(Addr a)
 {
    UInt idx = (a >> 16) & 0xffff;
    UInt off = (a >>  2) & 0x3fff;
-   EC_EIP ec = NULL_EC_EIP;
+   EC_IP ec = NULL_EC_IP;
 
    if (execontext_map[idx] != NULL)
       ec = execontext_map[idx]->execontext[off];
@@ -445,8 +445,8 @@ static Bool tlsIsDisjoint(const ThreadLifeSeg *tls,
 
 static inline UInt packTLS(ThreadLifeSeg *tls)
 {
-   SK_ASSERT(((UInt)tls & ((1 << STATE_BITS)-1)) == 0);
-   return ((UInt)tls) >> STATE_BITS;
+   SK_ASSERT(((UWord)tls & ((1 << STATE_BITS)-1)) == 0);
+   return ((UWord)tls) >> STATE_BITS;
 }
 
 static inline ThreadLifeSeg *unpackTLS(UInt i)
@@ -562,7 +562,7 @@ static __inline__
 void init_virgin_sword(Addr a)
 {
    if (clo_execontext != EC_None)
-      setExeContext(a, NULL_EC_EIP);
+      setExeContext(a, NULL_EC_IP);
    set_sword(a, virgin_sword);
 }
 
@@ -596,8 +596,6 @@ void init_magically_inited_sword(Addr a)
 {
    shadow_word sword;
 
-   sk_assert(VG_INVALID_THREADID == VG_(get_current_tid)());
-
    sword = SW(Vge_Virgin, TID_INDICATING_NONVIRGIN);
 
    set_sword(a, virgin_sword);
@@ -656,8 +654,8 @@ static inline UInt packLockSet(const LockSet *p)
 {
    UInt id;
 
-   SK_ASSERT(((UInt)p & ((1 << STATE_BITS)-1)) == 0);
-   id = ((UInt)p) >> STATE_BITS;
+   SK_ASSERT(((UWord)p & ((1 << STATE_BITS)-1)) == 0);
+   id = ((UWord)p) >> STATE_BITS;
 
    return id;
 }
@@ -710,7 +708,7 @@ static UInt hash_LockSet_w_wo(const LockSet *ls,
       }
 
       hash = ROTL(hash, 17);
-      hash ^= (UInt)mx->mutexp;
+      hash ^= mx->mutexp;
    }
 
    return hash % LOCKSET_HASH_SZ;
@@ -1348,7 +1346,7 @@ static void pp_all_mutexes()
 /* find or create a Mutex for a program's mutex use */
 static Mutex *get_mutex(Addr mutexp)
 {
-   UInt bucket = ((UInt)mutexp) % M_MUTEX_HASHSZ;
+   UInt bucket = mutexp % M_MUTEX_HASHSZ;
    Mutex *mp;
    
    for(mp = mutex_hash[bucket]; mp != NULL; mp = mp->next)
@@ -1585,7 +1583,7 @@ Bool cleanmx(Mutex *mx) {
 }
 
 static
-void set_address_range_state ( Addr a, UInt len /* in bytes */, 
+void set_address_range_state ( Addr a, SizeT len /* in bytes */, 
                                VgeInitStatus status )
 {
    Addr end;
@@ -1660,19 +1658,19 @@ void set_address_range_state ( Addr a, UInt len /* in bytes */,
 }
 
 
-static void make_segment_readable ( Addr a, UInt len )
+static void make_segment_readable ( Addr a, SizeT len )
 {
    //PROF_EVENT(??);    PPP
    set_address_range_state ( a, len, Vge_SegmentInit );
 }
 
-static void make_writable ( Addr a, UInt len )
+static void make_writable ( Addr a, SizeT len )
 {
    //PROF_EVENT(36);  PPP
    set_address_range_state( a, len, Vge_VirginInit );
 }
 
-static void make_readable ( Addr a, UInt len )
+static void make_readable ( Addr a, SizeT len )
 {
    //PROF_EVENT(37);  PPP
    set_address_range_state( a, len, Vge_VirginInit );
@@ -1680,7 +1678,7 @@ static void make_readable ( Addr a, UInt len )
 
 
 /* Block-copy states (needed for implementing realloc()). */
-static void copy_address_range_state(Addr src, Addr dst, UInt len)
+static void copy_address_range_state(Addr src, Addr dst, SizeT len)
 {
    UInt i;
 
@@ -1693,25 +1691,25 @@ static void copy_address_range_state(Addr src, Addr dst, UInt len)
 }
 
 // SSS: put these somewhere better
-static void eraser_mem_read (Addr a, UInt data_size, ThreadId tid);
-static void eraser_mem_write(Addr a, UInt data_size, ThreadId tid);
+static void eraser_mem_read (Addr a, SizeT data_size, ThreadId tid);
+static void eraser_mem_write(Addr a, SizeT data_size, ThreadId tid);
 
 static void eraser_mem_help_read_1(Addr a) REGPARM(1);
 static void eraser_mem_help_read_2(Addr a) REGPARM(1);
 static void eraser_mem_help_read_4(Addr a) REGPARM(1);
-static void eraser_mem_help_read_N(Addr a, UInt size) REGPARM(2);
+static void eraser_mem_help_read_N(Addr a, SizeT size) REGPARM(2);
 
 static void eraser_mem_help_write_1(Addr a, UInt val) REGPARM(2);
 static void eraser_mem_help_write_2(Addr a, UInt val) REGPARM(2);
 static void eraser_mem_help_write_4(Addr a, UInt val) REGPARM(2);
-static void eraser_mem_help_write_N(Addr a, UInt size) REGPARM(2);
+static void eraser_mem_help_write_N(Addr a, SizeT size) REGPARM(2);
 
 static void bus_lock(void);
 static void bus_unlock(void);
 
 static
 void eraser_pre_mem_read(CorePart part, ThreadId tid,
-                         Char* s, UInt base, UInt size )
+                         Char* s, Addr base, SizeT size )
 {
    if (tid > 50) { VG_(printf)("pid = %d, s = `%s`, part = %d\n", tid, s, part); VG_(skin_panic)("a");}
    eraser_mem_read(base, size, tid);
@@ -1719,14 +1717,14 @@ void eraser_pre_mem_read(CorePart part, ThreadId tid,
 
 static
 void eraser_pre_mem_read_asciiz(CorePart part, ThreadId tid,
-                                Char* s, UInt base )
+                                Char* s, Addr base )
 {
    eraser_mem_read(base, VG_(strlen)((Char*)base), tid);
 }
 
 static
 void eraser_pre_mem_write(CorePart part, ThreadId tid,
-                          Char* s, UInt base, UInt size )
+                          Char* s, Addr base, SizeT size )
 {
    eraser_mem_write(base, size, tid);
 }
@@ -1734,7 +1732,7 @@ void eraser_pre_mem_write(CorePart part, ThreadId tid,
 
 
 static
-void eraser_new_mem_startup( Addr a, UInt len, Bool rr, Bool ww, Bool xx )
+void eraser_new_mem_startup( Addr a, SizeT len, Bool rr, Bool ww, Bool xx )
 {
    /* Ignore the permissions, just make it readable.  Seems to work... */
    make_segment_readable(a, len);
@@ -1742,7 +1740,7 @@ void eraser_new_mem_startup( Addr a, UInt len, Bool rr, Bool ww, Bool xx )
 
 
 static
-void eraser_new_mem_heap ( Addr a, UInt len, Bool is_inited )
+void eraser_new_mem_heap ( Addr a, SizeT len, Bool is_inited )
 {
    if (is_inited) {
       make_readable(a, len);
@@ -1752,7 +1750,7 @@ void eraser_new_mem_heap ( Addr a, UInt len, Bool is_inited )
 }
 
 static
-void eraser_set_perms (Addr a, UInt len,
+void eraser_set_perms (Addr a, SizeT len,
                        Bool rr, Bool ww, Bool xx)
 {
    if      (rr) make_readable(a, len);
@@ -1761,13 +1759,13 @@ void eraser_set_perms (Addr a, UInt len,
 }
 
 static
-void eraser_new_mem_stack_private(Addr a, UInt len)
+void eraser_new_mem_stack_private(Addr a, SizeT len)
 {
    set_address_range_state(a, len, Vge_NonVirginInit);
 }
 
 static
-void eraser_new_mem_stack(Addr a, UInt len)
+void eraser_new_mem_stack(Addr a, SizeT len)
 {
    set_address_range_state(a, len, Vge_VirginInit);
 }
@@ -1810,7 +1808,7 @@ UInt VG_(vg_malloc_redzone_szB) = 8;
    shadow chunk on the appropriate list, and set all memory
    protections correctly. */
 
-static void add_HG_Chunk ( ThreadId tid, Addr p, UInt size )
+static void add_HG_Chunk ( ThreadId tid, Addr p, SizeT size )
 {
    HG_Chunk* hc;
 
@@ -1825,7 +1823,7 @@ static void add_HG_Chunk ( ThreadId tid, Addr p, UInt size )
 
 /* Allocate memory and note change in memory available */
 static __inline__
-void* alloc_and_new_mem ( Int size, UInt alignment, Bool is_zeroed )
+void* alloc_and_new_mem ( SizeT size, SizeT alignment, Bool is_zeroed )
 {
    Addr p;
 
@@ -1842,27 +1840,27 @@ void* alloc_and_new_mem ( Int size, UInt alignment, Bool is_zeroed )
    return (void*)p;
 }
 
-void* SK_(malloc) ( Int n )
+void* SK_(malloc) ( SizeT n )
 {
    return alloc_and_new_mem ( n, VG_(clo_alignment), /*is_zeroed*/False );
 }
 
-void* SK_(__builtin_new) ( Int n )
+void* SK_(__builtin_new) ( SizeT n )
 {
    return alloc_and_new_mem ( n, VG_(clo_alignment), /*is_zeroed*/False );
 }
 
-void* SK_(__builtin_vec_new) ( Int n )
+void* SK_(__builtin_vec_new) ( SizeT n )
 {
    return alloc_and_new_mem ( n, VG_(clo_alignment), /*is_zeroed*/False );
 }
 
-void* SK_(memalign) ( Int align, Int n )
+void* SK_(memalign) ( SizeT align, SizeT n )
 {
    return alloc_and_new_mem ( n, align,              /*is_zeroed*/False );
 }
 
-void* SK_(calloc) ( Int nmemb, Int size )
+void* SK_(calloc) ( SizeT nmemb, SizeT size )
 {
    return alloc_and_new_mem ( nmemb*size, VG_(clo_alignment),
                               /*is_zeroed*/True );
@@ -1920,7 +1918,7 @@ void handle_free ( void* p )
    HG_Chunk*  hc;
    HG_Chunk** prev_chunks_next_ptr;
 
-   hc = (HG_Chunk*)VG_(HT_get_node) ( hg_malloc_list, (UInt)p,
+   hc = (HG_Chunk*)VG_(HT_get_node) ( hg_malloc_list, (UWord)p,
                                       (VgHashNode***)&prev_chunks_next_ptr );
    if (hc == NULL) {
       return;
@@ -1944,7 +1942,7 @@ void SK_(__builtin_vec_delete) ( void* p )
    handle_free(p);
 }
 
-void* SK_(realloc) ( void* p, Int new_size )
+void* SK_(realloc) ( void* p, SizeT new_size )
 {
    HG_Chunk  *hc;
    HG_Chunk **prev_chunks_next_ptr;
@@ -1952,7 +1950,7 @@ void* SK_(realloc) ( void* p, Int new_size )
    ThreadId   tid = VG_(get_current_or_recent_tid)();
 
    /* First try and find the block. */
-   hc = (HG_Chunk*)VG_(HT_get_node) ( hg_malloc_list, (UInt)p,
+   hc = (HG_Chunk*)VG_(HT_get_node) ( hg_malloc_list, (UWord)p,
                                        (VgHashNode***)&prev_chunks_next_ptr );
 
    if (hc == NULL) {
@@ -2044,8 +2042,8 @@ UCodeBlock* SK_(instrument) ( UCodeBlock* cb_in, Addr not_used )
 
    /* stackref[] is used for super-simple value tracking to keep note
       of which tempregs currently hold a value which is derived from
-      ESP or EBP, and is therefore likely stack-relative if used as
-      the address for LOAD or STORE. */
+      the stack pointer or frame pointer, and is therefore likely
+      stack-relative if used as the address for LOAD or STORE. */
    ntemps = VG_(get_num_temps)(cb);
    stackref = VG_(malloc)(sizeof(*stackref) * ntemps);
    VG_(memset)(stackref, 0, sizeof(*stackref) * ntemps);
@@ -2079,7 +2077,8 @@ UCodeBlock* SK_(instrument) ( UCodeBlock* cb_in, Addr not_used )
 	    sk_assert(u_in->val2 < ntemps);
 
 	    stackref[u_in->val2] = (u_in->size == 4 &&
-				    (u_in->val1 == R_ESP || u_in->val1 == R_EBP));
+				    (u_in->val1 == R_STACK_PTR ||
+                                     u_in->val1 == R_FRAME_PTR));
 	    VG_(copy_UInstr)(cb, u_in);
 	    break;
 
@@ -2315,7 +2314,7 @@ typedef
       /* Segment */
       const Char* filename;
       const Char* section;
-      /* True if is just-below %esp -- could be a gcc bug. */
+      /* True if is just-below the stack pointer -- could be a gcc bug. */
       Bool maybe_gcc;
       /* symbolic address description */
       Char *expr;
@@ -2337,7 +2336,7 @@ typedef
       shadow_word prevstate;
       /* MutexErr, LockGraphErr */
       Mutex      *mutex;
-      EC_EIP      lasttouched;
+      EC_IP      lasttouched;
       ThreadId    lasttid;
       /* LockGraphErr */
       const LockSet    *held_lockset;
@@ -2366,7 +2365,7 @@ void clear_HelgrindError ( HelgrindError* err_extra )
    err_extra->axskind    = ReadAxs;
    err_extra->size       = 0;
    err_extra->mutex      = NULL;
-   err_extra->lasttouched= NULL_EC_EIP;
+   err_extra->lasttouched= NULL_EC_IP;
    err_extra->lasttid    = VG_INVALID_THREADID;
    err_extra->prev_lockset = 0;
    err_extra->held_lockset = 0;
@@ -2402,7 +2401,7 @@ static void describe_addr ( Addr a, AddrInfo* ai )
 	  seg != NULL; 
 	  seg = VG_(next_seginfo)(seg)) {
 	 Addr base = VG_(seg_start)(seg);
-	 UInt size = VG_(seg_size)(seg);
+	 SizeT size = VG_(seg_size)(seg);
 	 const UChar *filename = VG_(seg_filename)(seg);
 
 	 if (a >= base && a < base+size) {
@@ -2576,7 +2575,7 @@ static void pp_AddrInfo ( Addr a, AddrInfo* ai )
             in Memcheck */
          if (ai->maybe_gcc) {
             VG_(message)(Vg_UserMsg, 
-               " Address %p is just below %%esp.  Possibly a bug in GCC/G++",
+               " Address %p is just below the stack pointer.  Possibly a bug in GCC/G++",
                a);
             VG_(message)(Vg_UserMsg, 
                "   v 2.96 or 3.0.X.  To suppress, use: --workaround-gcc296-bugs=yes");
@@ -2592,10 +2591,10 @@ static void pp_AddrInfo ( Addr a, AddrInfo* ai )
 	break;
       case Mallocd:
       case Freed: {
-         UInt delta;
+         SizeT delta;
          UChar* relative;
          if (ai->rwoffset < 0) {
-            delta    = (UInt)(- ai->rwoffset);
+            delta    = (SizeT)(- ai->rwoffset);
             relative = "before";
          } else if (ai->rwoffset >= ai->blksize) {
             delta    = ai->rwoffset - ai->blksize;
@@ -2605,8 +2604,8 @@ static void pp_AddrInfo ( Addr a, AddrInfo* ai )
             relative = "inside";
          }
 	 VG_(message)(Vg_UserMsg, 
-		      " Address %p is %d bytes %s a block of size %d %s by thread %d",
-		      a, delta, relative, 
+		      " Address %p is %llu bytes %s a block of size %d %s by thread %d",
+		      a, (ULong)delta, relative, 
 		      ai->blksize,
 		      ai->akind == Mallocd ? "alloc'd" : "freed",
 		      ai->lasttid);
@@ -2697,32 +2696,32 @@ void SK_(pp_SkinError) ( Error* err )
 	 VG_(message)(Vg_UserMsg, " Previous state: %s", msg);
 
       if (clo_execontext == EC_Some 
-          && extra->lasttouched.uu_ec_eip.eip != 0) {
+          && extra->lasttouched.uu_ec_ip.ip != 0) {
 	 Char file[100];
 	 UInt line;
-	 Addr eip = extra->lasttouched.uu_ec_eip.eip;
+	 Addr ip = extra->lasttouched.uu_ec_ip.ip;
 	 
 	 VG_(message)(Vg_UserMsg, " Word at %p last changed state from %s by thread %u",
 		      err_addr,
 		      pp_state(extra->lasttouched.state),
 		      unpackTLS(extra->lasttouched.tls)->tid);
 	 
-	 if (VG_(get_filename_linenum)(eip, file, sizeof(file), &line)) {
+	 if (VG_(get_filename_linenum)(ip, file, sizeof(file), &line)) {
 	    VG_(message)(Vg_UserMsg, "   at %p: %y (%s:%u)",
-			 eip, eip, file, line);
-	 } else if (VG_(get_objname)(eip, file, sizeof(file))) {
+			 ip, ip, file, line);
+	 } else if (VG_(get_objname)(ip, file, sizeof(file))) {
 	    VG_(message)(Vg_UserMsg, "   at %p: %y (in %s)",
-			 eip, eip, file);
+			 ip, ip, file);
 	 } else {
-	    VG_(message)(Vg_UserMsg, "   at %p: %y", eip, eip);
+	    VG_(message)(Vg_UserMsg, "   at %p: %y", ip, ip);
 	 }
       } else if (clo_execontext == EC_All 
-                 && extra->lasttouched.uu_ec_eip.ec != NULL) {
+                 && extra->lasttouched.uu_ec_ip.ec != NULL) {
 	 VG_(message)(Vg_UserMsg, " Word at %p last changed state from %s in tid %u",
 		      err_addr,
 		      pp_state(extra->lasttouched.state),
 		      unpackTLS(extra->lasttouched.tls)->tid);
-	 VG_(pp_ExeContext)(extra->lasttouched.uu_ec_eip.ec);
+	 VG_(pp_ExeContext)(extra->lasttouched.uu_ec_ip.ec);
       }
       break;
    }
@@ -2733,9 +2732,9 @@ void SK_(pp_SkinError) ( Error* err )
 		   VG_(get_error_address)(err),
 		   VG_(get_error_string)(err));
       VG_(pp_ExeContext)( VG_(get_error_where)(err) );
-      if (extra->lasttouched.uu_ec_eip.ec != NULL) {
+      if (extra->lasttouched.uu_ec_ip.ec != NULL) {
 	 VG_(message)(Vg_UserMsg, " last touched by thread %d", extra->lasttid);
-	 VG_(pp_ExeContext)(extra->lasttouched.uu_ec_eip.ec);
+	 VG_(pp_ExeContext)(extra->lasttouched.uu_ec_ip.ec);
       }
       pp_AddrInfo(VG_(get_error_address)(err), &extra->addrinfo);
       break;
@@ -3023,17 +3022,17 @@ static void eraser_mem_read_word(Addr a, ThreadId tid)
 
   done:
    if (clo_execontext != EC_None && statechange) {
-      EC_EIP eceip;
+      EC_IP ecip;
 
       if (clo_execontext == EC_Some)
-	 eceip = EIP(VG_(get_EIP)(tid), prevstate, tls);
+	 ecip = IP(VG_(get_EIP)(tid), prevstate, tls);
       else
-	 eceip = EC(VG_(get_ExeContext)(tid), prevstate, tls);
-      setExeContext(a, eceip);
+	 ecip = EC(VG_(get_ExeContext)(tid), prevstate, tls);
+      setExeContext(a, ecip);
    }
 }
 
-static void eraser_mem_read(Addr a, UInt size, ThreadId tid)
+static void eraser_mem_read(Addr a, SizeT size, ThreadId tid)
 {
    Addr end;
 
@@ -3128,17 +3127,17 @@ static void eraser_mem_write_word(Addr a, ThreadId tid)
 
   done:
    if (clo_execontext != EC_None && statechange) {
-      EC_EIP eceip;
+      EC_IP ecip;
 
       if (clo_execontext == EC_Some)
-	 eceip = EIP(VG_(get_EIP)(tid), prevstate, tls);
+	 ecip = IP(VG_(get_EIP)(tid), prevstate, tls);
       else
-	 eceip = EC(VG_(get_ExeContext)(tid), prevstate, tls);
-      setExeContext(a, eceip);
+	 ecip = EC(VG_(get_ExeContext)(tid), prevstate, tls);
+      setExeContext(a, ecip);
    }
 }
 
-static void eraser_mem_write(Addr a, UInt size, ThreadId tid)
+static void eraser_mem_write(Addr a, SizeT size, ThreadId tid)
 {
    Addr     end;
 
@@ -3153,42 +3152,42 @@ static void eraser_mem_write(Addr a, UInt size, ThreadId tid)
 
 REGPARM(1) static void eraser_mem_help_read_1(Addr a)
 {
-   eraser_mem_read(a, 1, VG_(get_current_tid)());
+   eraser_mem_read(a, 1, VG_(get_VCPU_tid)());
 }
 
 REGPARM(1) static void eraser_mem_help_read_2(Addr a)
 {
-   eraser_mem_read(a, 2, VG_(get_current_tid)());
+   eraser_mem_read(a, 2, VG_(get_VCPU_tid)());
 }
 
 REGPARM(1) static void eraser_mem_help_read_4(Addr a)
 {
-   eraser_mem_read(a, 4, VG_(get_current_tid)());
+   eraser_mem_read(a, 4, VG_(get_VCPU_tid)());
 }
 
-REGPARM(2) static void eraser_mem_help_read_N(Addr a, UInt size)
+REGPARM(2) static void eraser_mem_help_read_N(Addr a, SizeT size)
 {
-   eraser_mem_read(a, size, VG_(get_current_tid)());
+   eraser_mem_read(a, size, VG_(get_VCPU_tid)());
 }
 
 REGPARM(2) static void eraser_mem_help_write_1(Addr a, UInt val)
 {
    if (*(UChar *)a != val)
-      eraser_mem_write(a, 1, VG_(get_current_tid)());
+      eraser_mem_write(a, 1, VG_(get_VCPU_tid)());
 }
 REGPARM(2) static void eraser_mem_help_write_2(Addr a, UInt val)
 {
    if (*(UShort *)a != val)
-      eraser_mem_write(a, 2, VG_(get_current_tid)());
+      eraser_mem_write(a, 2, VG_(get_VCPU_tid)());
 }
 REGPARM(2) static void eraser_mem_help_write_4(Addr a, UInt val)
 {
    if (*(UInt *)a != val)
-      eraser_mem_write(a, 4, VG_(get_current_tid)());
+      eraser_mem_write(a, 4, VG_(get_VCPU_tid)());
 }
-REGPARM(2) static void eraser_mem_help_write_N(Addr a, UInt size)
+REGPARM(2) static void eraser_mem_help_write_N(Addr a, SizeT size)
 {
-   eraser_mem_write(a, size, VG_(get_current_tid)());
+   eraser_mem_write(a, size, VG_(get_VCPU_tid)());
 }
 
 static void hg_thread_create(ThreadId parent, ThreadId child)
@@ -3217,14 +3216,14 @@ static Int __BUS_HARDWARE_LOCK__;
 
 static void bus_lock(void)
 {
-   ThreadId tid = VG_(get_current_tid)();
+   ThreadId tid = VG_(get_VCPU_tid)();
    eraser_pre_mutex_lock(tid, &__BUS_HARDWARE_LOCK__);
    eraser_post_mutex_lock(tid, &__BUS_HARDWARE_LOCK__);
 }
 
 static void bus_unlock(void)
 {
-   ThreadId tid = VG_(get_current_tid)();
+   ThreadId tid = VG_(get_VCPU_tid)();
    eraser_post_mutex_unlock(tid, &__BUS_HARDWARE_LOCK__);
 }
 
@@ -3232,7 +3231,7 @@ static void bus_unlock(void)
 /*--- Client requests                                              ---*/
 /*--------------------------------------------------------------------*/
 
-Bool SK_(handle_client_request)(ThreadId tid, UInt *args, UInt *ret)
+Bool SK_(handle_client_request)(ThreadId tid, UWord *args, UWord *ret)
 {
    if (!VG_IS_SKIN_USERREQ('H','G',args[0]))
       return False;
@@ -3269,7 +3268,7 @@ void SK_(pre_clo_init)(void)
    VG_(details_version)         (NULL);
    VG_(details_description)     ("a data race detector");
    VG_(details_copyright_author)(
-      "Copyright (C) 2002-2004, and GNU GPL'd, by Nicholas Nethercote et al.");
+      "Copyright (C) 2002-2005, and GNU GPL'd, by Nicholas Nethercote et al.");
    VG_(details_bug_reports_to)  (VG_BUGS_TO);
    VG_(details_avg_translation_sizeB) ( 115 );
 
@@ -3308,19 +3307,6 @@ void SK_(pre_clo_init)(void)
    VG_(init_post_mutex_lock)      (& eraser_post_mutex_lock);
    VG_(init_post_mutex_unlock)    (& eraser_post_mutex_unlock);
 
-   VG_(register_compact_helper)((Addr) & eraser_mem_help_read_1);
-   VG_(register_compact_helper)((Addr) & eraser_mem_help_read_2);
-   VG_(register_compact_helper)((Addr) & eraser_mem_help_read_4);
-   VG_(register_noncompact_helper)((Addr) & eraser_mem_help_read_N);
-
-   VG_(register_compact_helper)((Addr) & eraser_mem_help_write_1);
-   VG_(register_compact_helper)((Addr) & eraser_mem_help_write_2);
-   VG_(register_compact_helper)((Addr) & eraser_mem_help_write_4);
-   VG_(register_noncompact_helper)((Addr) & eraser_mem_help_write_N);
-
-   VG_(register_noncompact_helper)((Addr) & bus_lock);
-   VG_(register_noncompact_helper)((Addr) & bus_unlock);
-
    for(i = 0; i < LOCKSET_HASH_SZ; i++)
       lockset_hash[i] = NULL;
 
@@ -3373,7 +3359,7 @@ void SK_(print_debug_usage)(void)
 
 void SK_(post_clo_init)(void)
 {
-   void (*stack_tracker)(Addr a, UInt len);
+   void (*stack_tracker)(Addr a, SizeT len);
    
    if (clo_execontext) {
       execontext_map = VG_(malloc)(sizeof(ExeContextMap *) * 65536);
