@@ -8,7 +8,7 @@
    This file is part of MemCheck, a heavyweight Valgrind tool for
    detecting memory errors.
 
-   Copyright (C) 2000-2004 Julian Seward 
+   Copyright (C) 2000-2005 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -36,8 +36,8 @@
    ------------------------------------------------------------------ */
 
 /* Compare this with the restrictions on core instructions in
-   vg_translate.c:VG_(saneUInstr)().  Everything general said there applies
-   here too.
+   vg_translate.c:is_sane_UInstr().  Everything general said there
+   applies here too.
 */
 Bool SK_(sane_XUInstr)(Bool beforeRA, Bool beforeLiveness, UInstr* u)
 {
@@ -57,7 +57,6 @@ Bool SK_(sane_XUInstr)(Bool beforeRA, Bool beforeLiveness, UInstr* u)
 #  define Ls1 (u->tag1 == Lit16)
 #  define Ls3 (u->tag3 == Lit16)
 #  define TRL1 (TR1 || L1)
-#  define TRA1 (TR1 || A1)
 #  define N2  (u->tag2 == NoValue)
 #  define N3  (u->tag3 == NoValue)
 #  define COND0    (u->cond         == 0)
@@ -89,7 +88,7 @@ Bool SK_(sane_XUInstr)(Bool beforeRA, Bool beforeLiveness, UInstr* u)
    case GETVF: 
    case PUTVF:  return LIT0 && SZ0 && CC0 &&  TR1 &&  N2 &&  N3 && XOTHER;
    case TESTV: 
-   case SETV:   return LIT0 && SZj && CC0 && TRA1 &&  N2 &&  N3 && XOTHER;
+   case SETV:   return LIT0 && SZj && CC0 &&  TR1 &&  N2 &&  N3 && XOTHER;
    case TAG1:   return LIT0 && SZ0 && CC0 &&  TR1 &&  N2 && Ls3 && XOTHER;
    case TAG2:   return LIT0 && SZ0 && CC0 &&  TR1 && TR2 && Ls3 && XOTHER;
    default:
@@ -111,7 +110,6 @@ Bool SK_(sane_XUInstr)(Bool beforeRA, Bool beforeLiveness, UInstr* u)
 #  undef Ls1
 #  undef Ls3
 #  undef TRL1
-#  undef TRA1
 #  undef N2
 #  undef N3
 #  undef COND0
@@ -560,7 +558,7 @@ static UCodeBlock* memcheck_instrument ( UCodeBlock* cb_in )
 
          case GETSEG:
             sk_assert(u_in->tag2 == TempReg);
-            uInstr1(cb, SETV,  2, TempReg, SHADOW(u_in->val2));
+            uInstr1(cb, SETV,  u_in->size, TempReg, SHADOW(u_in->val2));
             VG_(copy_UInstr)(cb, u_in);
             break;
 
@@ -575,7 +573,7 @@ static UCodeBlock* memcheck_instrument ( UCodeBlock* cb_in )
             still addressible.  Hence the optionalisation of the V
             check.  15 Dec 02: optionalisation removed, since it no
             longer makes much sense given we also have an addrcheck
-            skin.
+            tool.
 
             The LOADV/STOREV does an addressibility check for the
             address. */
@@ -1075,6 +1073,25 @@ static UCodeBlock* memcheck_instrument ( UCodeBlock* cb_in )
             break;
          }
 
+         case MMX2a1_MemRd: {
+            Int t_size = INVALID_TEMPREG;
+
+            sk_assert(u_in->size == 8);
+
+            sk_assert(u_in->tag3 == TempReg);
+            uInstr1(cb, TESTV, 4, TempReg, SHADOW(u_in->val3));
+            uInstr1(cb, SETV,  4, TempReg, SHADOW(u_in->val3));
+
+            t_size = newTemp(cb);
+            uInstr2(cb, MOV,   4, Literal, 0, TempReg, t_size);
+            uLiteral(cb, u_in->size);
+            uInstr2(cb, CCALL, 0, TempReg, u_in->val3, TempReg, t_size);
+            uCCall(cb, (Addr) & MC_(fpu_read_check), 2, 2, False);
+            
+            VG_(copy_UInstr)(cb, u_in);
+            break;
+         }
+
 	 /* SSE ins referencing scalar integer registers */
          case SSE2g_RegWr:
          case SSE2g1_RegWr:
@@ -1093,7 +1110,8 @@ static UCodeBlock* memcheck_instrument ( UCodeBlock* cb_in )
             }
 
             /* Is it a read ?  Better check the V bits right now. */
-            if ( u_in->opcode == SSE3e_RegRd
+            if ( u_in->opcode == SSE2e1_RegRd
+                 || u_in->opcode == SSE3e_RegRd
                  || u_in->opcode == SSE3e1_RegRd )
 	       uInstr1(cb, TESTV, u_in->size, 
                            TempReg, SHADOW(u_in->val3));
@@ -1158,12 +1176,26 @@ static UCodeBlock* memcheck_instrument ( UCodeBlock* cb_in )
             VG_(copy_UInstr)(cb, u_in);
 	    break;
          }
-
-         /* For FPU, MMX and SSE insns not referencing memory, just
-            copy thru. */
+         
+         /* For MMX and SSE insns not referencing memory, just
+            make sure the eflags are defined if the instruction
+            read them, and make them defined it it writes them. */
          case SSE5: case SSE4: case SSE3:
          case MMX1: case MMX2: case MMX3:
-         case FPU: 
+         case FPU:
+            if (u_in->flags_r != FlagsEmpty) {
+               qt = create_GETVF(cb, 0);
+               uInstr1(cb, TESTV, 0, TempReg, qt);
+               /* qt should never be referred to again.  Nevertheless
+                  ... */
+               uInstr1(cb, SETV, 0, TempReg, qt);
+            }
+            if (u_in->flags_w != FlagsEmpty) {
+               qd = newTemp(cb);
+               uInstr2(cb, MOV, 4, Literal, 0, TempReg, qd);
+               uLiteral(cb, 0);
+               create_PUTVF(cb, 0, qd);
+            }
             VG_(copy_UInstr)(cb, u_in);
             break;
 
